package com.logicbig.example;


import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;

public class EmployeeClient {
    private static Client client = ClientBuilder.newClient();
    private static WebTarget baseTarget = client.target("http://localhost:8080/employees");
    private static List<String> deptList = Arrays.asList("Admin", "IT", "Sell");

    public static void main(String[] args) {
        createEmployees();
        listEmployees();
        deleteEmployeeByIds(2, 4);
        Employee e = getEmployee(3);
        e.setDept("Marketing");
        updateEmployee(e);
        listEmployees();
    }

    private static Employee getEmployee(long id) {
        System.out.printf("-- getting employee by id: %s --%n", id);
        Employee e = baseTarget.path(Long.toString(id))
                               .request(MediaType.APPLICATION_XML)
                               .get(Employee.class);
        System.out.println("employee retrieved: " + e);
        return e;
    }

    private static void updateEmployee(Employee e) {
        System.out.println("-- updating employee --");
        System.out.println(e);
        String response = baseTarget.request(MediaType.APPLICATION_XML)
                                    .post(Entity.xml(e), String.class);
        System.out.println(response);
    }

    private static void deleteEmployeeByIds(long... ids) {
        System.out.printf("-- deleting employees: %s --%n",
                Arrays.toString(ids));
        for (long id : ids) {
            WebTarget target = baseTarget.path(Long.toString(id));
            Response r = target.request()
                               .delete();
            System.out.println(r.readEntity(String.class));
        }
    }

    private static void listEmployees() {
        System.out.println("-- getting employee list --");
        List<Employee> employees = baseTarget.request().get(
                new GenericType<List<Employee>>() {
                });
        employees.forEach(System.out::println);
    }

    private static void createEmployees() {
        System.out.println("-- making PUT requests --");
        for (int i = 1; i <= 5; i++) {
            Employee e = new Employee();
            e.setName("Employee" + i);
            int index = ThreadLocalRandom.current()
                                         .nextInt(0, 3);
            e.setDept(deptList.get(index));

            WebTarget target = baseTarget.path(Integer.toString(i));
            Response r = target.request()
                               .put(Entity.entity(e, MediaType.APPLICATION_XML));
            System.out.println(r.readEntity(String.class));
        }
    }
}