package com.logicbig.example;

import java.util.*;

public enum InMemoryEmployeeService implements EmployeeService {
    Instance;
    private Set<Employee> employees = new HashSet<>();

    @Override
    public String create(Employee employee, long newId) {
        employee.setId(Long.valueOf(newId));
        if (employees.contains(employee)) {
            throw new RuntimeException("Employee with id already exists: "
                    + newId);
        }
        employees.add(employee);
        return "Msg: employee created for id: " + newId;
    }

    @Override
    public String delete(long id) {
        boolean b = employees.removeIf(e -> Long.compare(id, e.getId()) == 0);
        return b ? "Msg: employee deleted with id: " + id :
                "No employee found for id: " + id;
    }

    @Override
    public String update(Employee employee) {
        if (employee.getId() == 0) {
            throw new RuntimeException("Id must be set for Employee update");
        }
        if (employees.contains(employee)) {
            employees.remove(employee);
            employees.add(employee);
            return "employee updated : " + employee;
        }

        return "No Employee exists: " + employee;
    }

    @Override
    public List<Employee> getEmployees() {
        return new ArrayList<>(employees);
    }

    @Override
    public Employee getEmployeeById(long id) {
        Optional<Employee> opt = employees.stream()
                                          .filter(e -> e.getId() == id)
                                          .findAny();
        if (opt.isPresent()) {
            return opt.get();
        }
        return null;
    }
}